@@ -22,23 +22,28 @@ import errno
 import hashlib
 import logging
 import os
-import urlparse
+import stat
 
-from oslo.config import cfg
+import jsonschema
+from oslo_config import cfg
+from oslo_serialization import jsonutils
+from oslo_utils import excutils
+from oslo_utils import units
+from six.moves import urllib
 
 import glance_store
+from glance_store import capabilities
 from glance_store.common import utils
 import glance_store.driver
 from glance_store import exceptions
-from glance_store.i18n import _
+from glance_store import i18n
 import glance_store.location
-from glance_store.openstack.common import excutils
-from glance_store.openstack.common import jsonutils
-from glance_store.openstack.common import processutils
-from glance_store.openstack.common import units
 
 
 LOG = logging.getLogger(__name__)
+_ = i18n._
+_LE = i18n._LE
+_LW = i18n._LW
 
 _FILESYSTEM_CONFIGS = [
     cfg.StrOpt('filesystem_store_datadir',
@@ -51,7 +56,30 @@ _FILESYSTEM_CONFIGS = [
                help=_("The path to a file which contains the "
                       "metadata to be returned with any location "
                       "associated with this store.  The file must "
-                      "contain a valid JSON dict."))]
+                      "contain a valid JSON object. The object should contain "
+                      "the keys 'id' and 'mountpoint'. The value for both "
+                      "keys should be 'string'.")),
+    cfg.IntOpt('filesystem_store_file_perm',
+               default=0,
+               help=_("The required permission for created image file. "
+                      "In this way the user other service used, e.g. Nova, "
+                      "who consumes the image could be the exclusive member "
+                      "of the group that owns the files created. Assigning "
+                      "it less then or equal to zero means don't change the "
+                      "default permission of the file. This value will be "
+                      "decoded as an octal digit."))]
+
+MULTI_FILESYSTEM_METADATA_SCHEMA = {
+    "type": "array",
+    "items": {
+        "type": "object",
+        "properties": {
+            "id": {"type": "string"},
+            "mountpoint": {"type": "string"}
+        },
+        "required": ["id", "mountpoint"],
+    }
+}
 
 
 class StoreLocation(glance_store.location.StoreLocation):
@@ -70,7 +98,7 @@ class StoreLocation(glance_store.location.StoreLocation):
         in the URL are interpreted differently in Python 2.6.1+ than prior
         versions of Python.
         """
-        pieces = urlparse.urlparse(uri)
+        pieces = urllib.parse.urlparse(uri)
         assert pieces.scheme in ('file', 'filesystem')
         self.scheme = pieces.scheme
         path = (pieces.netloc + pieces.path).strip()
@@ -88,25 +116,34 @@ class ChunkedFile(object):
     something that can iterate over a large file
     """
 
-    def __init__(self, filepath, offset=0, chunk_size=None, partial=False):
-        self.partial = partial
+    def __init__(self, filepath, offset=0, chunk_size=4096,
+                 partial_length=None):
         self.filepath = filepath
         self.chunk_size = chunk_size
+        self.partial_length = partial_length
+        self.partial = self.partial_length is not None
         self.fp = open(self.filepath, 'rb')
         if offset:
             self.fp.seek(offset)
 
     def __iter__(self):
-        """Return an iterator over the image file"""
+        """Return an iterator over the image file."""
         try:
             if self.fp:
                 while True:
-                    chunk = self.fp.read(self.chunk_size)
+                    if self.partial:
+                        size = min(self.chunk_size, self.partial_length)
+                    else:
+                        size = self.chunk_size
+
+                    chunk = self.fp.read(size)
                     if chunk:
                         yield chunk
 
                         if self.partial:
-                            break
+                            self.partial_length -= len(chunk)
+                            if self.partial_length <= 0:
+                                break
                     else:
                         break
         finally:
@@ -121,9 +158,13 @@ class ChunkedFile(object):
 
 class Store(glance_store.driver.Store):
 
+    _CAPABILITIES = (capabilities.BitMasks.READ_RANDOM |
+                     capabilities.BitMasks.WRITE_ACCESS |
+                     capabilities.BitMasks.DRIVER_REUSABLE)
     OPTIONS = _FILESYSTEM_CONFIGS
     READ_CHUNKSIZE = 64 * units.Ki
     WRITE_CHUNKSIZE = READ_CHUNKSIZE
+    FILESYSTEM_STORE_METADATA = None
 
     def get_schemes(self):
         return ('file', 'filesystem')
@@ -142,6 +183,35 @@ class Store(glance_store.driver.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name="filesystem", reason=msg)
 
+    def _set_exec_permission(self, datadir):
+        """
+        Set the execution permission of owner-group and/or other-users to
+        image directory if the image file which contained needs relevant
+        access permissions.
+
+        :datadir is a directory path in which glance writes image files.
+        """
+
+        if self.conf.glance_store.filesystem_store_file_perm <= 0:
+            return
+
+        try:
+            mode = os.stat(datadir)[stat.ST_MODE]
+            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
+                       8)
+            if perm & stat.S_IRWXO > 0:
+                if not mode & stat.S_IXOTH:
+                    # chmod o+x
+                    mode |= stat.S_IXOTH
+                    os.chmod(datadir, mode)
+            if perm & stat.S_IRWXG > 0:
+                if not mode & stat.S_IXGRP:
+                    # chmod g+x
+                    os.chmod(datadir, mode | stat.S_IXGRP)
+        except (IOError, OSError):
+            LOG.warn(_LW("Unable to set execution permission of owner-group "
+                         "and/or other-users to datadir: %s") % datadir)
+
     def _create_image_directories(self, directory_paths):
         """
         Create directories to write image files if
@@ -153,6 +223,7 @@ class Store(glance_store.driver.Store):
         for datadir in directory_paths:
             if os.path.exists(datadir):
                 self._check_write_permission(datadir)
+                self._set_exec_permission(datadir)
             else:
                 msg = _("Directory to write image files does not exist "
                         "(%s). Creating.") % datadir
@@ -160,6 +231,7 @@ class Store(glance_store.driver.Store):
                 try:
                     os.makedirs(datadir)
                     self._check_write_permission(datadir)
+                    self._set_exec_permission(datadir)
                 except (IOError, OSError):
                     if os.path.exists(datadir):
                         # NOTE(markwash): If the path now exists, some other
@@ -167,12 +239,54 @@ class Store(glance_store.driver.Store):
                         # But it doesn't hurt, so we can safely ignore
                         # the error.
                         self._check_write_permission(datadir)
+                        self._set_exec_permission(datadir)
                         continue
                     reason = _("Unable to create datadir: %s") % datadir
                     LOG.error(reason)
                     raise exceptions.BadStoreConfiguration(
                         store_name="filesystem", reason=reason)
 
+    def _validate_metadata(self, metadata_file):
+        """Validate metadata against json schema.
+
+        If metadata is valid then cache metadata and use it when
+        creating new image.
+
+        :param metadata_file: JSON metadata file path
+        :raises: BadStoreConfiguration exception if metadata is not valid.
+        """
+        try:
+            with open(metadata_file, 'rb') as fptr:
+                metadata = jsonutils.load(fptr)
+
+            if isinstance(metadata, dict):
+                # If metadata is of type dictionary
+                # i.e. - it contains only one mountpoint
+                # then convert it to list of dictionary.
+                metadata = [metadata]
+
+            # Validate metadata against json schema
+            jsonschema.validate(metadata, MULTI_FILESYSTEM_METADATA_SCHEMA)
+            glance_store.check_location_metadata(metadata)
+            self.FILESYSTEM_STORE_METADATA = metadata
+        except (jsonschema.exceptions.ValidationError,
+                exceptions.BackendException, ValueError) as vee:
+            reason = _('The JSON in the metadata file %(file)s is '
+                       'not valid and it can not be used: '
+                       '%(vee)s.') % dict(file=metadata_file,
+                                          vee=utils.exception_to_str(vee))
+            LOG.error(reason)
+            raise exceptions.BadStoreConfiguration(
+                store_name="filesystem", reason=reason)
+        except IOError as ioe:
+            reason = _('The path for the metadata file %(file)s could '
+                       'not be accessed: '
+                       '%(ioe)s.') % dict(file=metadata_file,
+                                          ioe=utils.exception_to_str(ioe))
+            LOG.error(reason)
+            raise exceptions.BadStoreConfiguration(
+                store_name="filesystem", reason=reason)
+
     def configure_add(self):
         """
         Configure the Store to use the stored configuration options
@@ -197,6 +311,19 @@ class Store(glance_store.driver.Store):
             raise exceptions.BadStoreConfiguration(store_name="filesystem",
                                                    reason=reason)
 
+        if self.conf.glance_store.filesystem_store_file_perm > 0:
+            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
+                       8)
+            if not perm & stat.S_IRUSR:
+                reason = _LE("Specified an invalid "
+                             "'filesystem_store_file_perm' option which "
+                             "could make image file to be unaccessible by "
+                             "glance service.")
+                LOG.error(reason)
+                reason = _("Invalid 'filesystem_store_file_perm' option.")
+                raise exceptions.BadStoreConfiguration(store_name="filesystem",
+                                                       reason=reason)
+
         self.multiple_datadirs = False
         directory_paths = set()
         if self.conf.glance_store.filesystem_store_datadir:
@@ -208,17 +335,24 @@ class Store(glance_store.driver.Store):
             for datadir in self.conf.glance_store.filesystem_store_datadirs:
                 (datadir_path,
                  priority) = self._get_datadir_path_and_priority(datadir)
-                self._check_directory_paths(datadir_path, directory_paths)
+                priority_paths = self.priority_data_map.setdefault(
+                    int(priority), [])
+                self._check_directory_paths(datadir_path, directory_paths,
+                                            priority_paths)
                 directory_paths.add(datadir_path)
-                self.priority_data_map.setdefault(int(priority),
-                                                  []).append(datadir_path)
+                priority_paths.append(datadir_path)
 
             self.priority_list = sorted(self.priority_data_map,
                                         reverse=True)
 
         self._create_image_directories(directory_paths)
 
-    def _check_directory_paths(self, datadir_path, directory_paths):
+        metadata_file = self.conf.glance_store.filesystem_store_metadata_file
+        if metadata_file:
+            self._validate_metadata(metadata_file)
+
+    def _check_directory_paths(self, datadir_path, directory_paths,
+                               priority_paths):
         """
         Checks if directory_path is already present in directory_paths.
 
@@ -232,9 +366,15 @@ class Store(glance_store.driver.Store):
                      "multiple times in filesystem_store_datadirs "
                      "option of filesystem configuration") %
                    {'datadir_path': datadir_path})
-            LOG.exception(msg)
-            raise exceptions.BadStoreConfiguration(
-                store_name="filesystem", reason=msg)
+
+            # If present with different priority it's a bad configuration
+            if datadir_path not in priority_paths:
+                LOG.exception(msg)
+                raise exceptions.BadStoreConfiguration(
+                    store_name="filesystem", reason=msg)
+
+            # Present with same prio (exact duplicate) only deserves a warning
+            LOG.warning(msg)
 
     def _get_datadir_path_and_priority(self, datadir):
         """
@@ -248,7 +388,7 @@ class Store(glance_store.driver.Store):
                empty directory path is specified.
         """
         priority = 0
-        parts = map(lambda x: x.strip(), datadir.rsplit(":", 1))
+        parts = [part.strip() for part in datadir.rsplit(":", 1)]
         datadir_path = parts[0]
         if len(parts) == 2 and parts[1]:
             priority = parts[1]
@@ -277,36 +417,43 @@ class Store(glance_store.driver.Store):
         filesize = os.path.getsize(filepath)
         return filepath, filesize
 
-    def _get_metadata(self):
-        metadata_file = self.conf.glance_store.filesystem_store_metadata_file
+    def _get_metadata(self, filepath):
+        """Return metadata dictionary.
 
-        if metadata_file is None:
-            return {}
+        If metadata is provided as list of dictionaries then return
+        metadata as dictionary containing 'id' and 'mountpoint'.
 
-        try:
-            with open(metadata_file, 'r') as fptr:
-                metadata = jsonutils.load(fptr)
+        If there are multiple nfs directories (mountpoints) configured
+        for glance, then we need to create metadata JSON file as list
+        of dictionaries containing all mountpoints with unique id.
+        But Nova will not be able to find in which directory (mountpoint)
+        image is present if we store list of dictionary(containing mountpoints)
+        in glance image metadata. So if there are multiple mountpoints then
+        we will return dict containing exact mountpoint where image is stored.
 
-            glance_store.check_location_metadata(metadata)
-            return metadata
-        except exceptions.BackendException as bee:
-            LOG.error(_('The JSON in the metadata file %(file)s could not '
-                        'be used: %(bee)s  An empty dictionary will be '
-                        'returned to the client.')
-                      % dict(file=metadata_file, bee=str(bee)))
-            return {}
-        except IOError as ioe:
-            LOG.error(_('The path for the metadata file %(file)s could not be '
-                        'opened: %(io)s  An empty dictionary will be returned '
-                        'to the client.')
-                      % dict(file=metadata_file, io=ioe))
-            return {}
-        except Exception as ex:
-            LOG.exception(_('An error occurred processing the storage systems '
-                            'meta data file: %s.  An empty dictionary will be '
-                            'returned to the client.') % str(ex))
-            return {}
+        If image path does not start with any of the 'mountpoint' provided
+        in metadata JSON file then error is logged and empty
+        dictionary is returned.
 
+        :param filepath: Path of image on store
+        :returns: metadata dictionary
+        """
+        if self.FILESYSTEM_STORE_METADATA:
+            for image_meta in self.FILESYSTEM_STORE_METADATA:
+                if filepath.startswith(image_meta['mountpoint']):
+                    return image_meta
+
+            reason = (_LE("The image path %(path)s does not match with "
+                          "any of the mountpoint defined in "
+                          "metadata: %(metadata)s. An empty dictionary "
+                          "will be returned to the client.")
+                      % dict(path=filepath,
+                             metadata=self.FILESYSTEM_STORE_METADATA))
+            LOG.error(reason)
+
+        return {}
+
+    @capabilities.check
     def get(self, location, offset=0, chunk_size=None, context=None):
         """
         Takes a `glance_store.location.Location` object that indicates
@@ -322,9 +469,9 @@ class Store(glance_store.driver.Store):
         LOG.debug(msg)
         return (ChunkedFile(filepath,
                             offset=offset,
-                            chunk_size=chunk_size or self.READ_CHUNKSIZE,
-                            partial=chunk_size is not None),
-                filesize)
+                            chunk_size=self.READ_CHUNKSIZE,
+                            partial_length=chunk_size),
+                chunk_size or filesize)
 
     def get_size(self, location, context=None):
         """
@@ -341,6 +488,7 @@ class Store(glance_store.driver.Store):
         LOG.debug(msg)
         return filesize
 
+    @capabilities.check
     def delete(self, location, context=None):
         """
         Takes a `glance_store.location.Location` object that indicates
@@ -359,7 +507,8 @@ class Store(glance_store.driver.Store):
                 LOG.debug(_("Deleting image at %(fn)s"), {'fn': fn})
                 os.unlink(fn)
             except OSError:
-                raise exceptions.Forbidden(_("You cannot delete file %s") % fn)
+                raise exceptions.Forbidden(
+                    message=(_("You cannot delete file %s") % fn))
         else:
             raise exceptions.NotFound(image=fn)
 
@@ -369,11 +518,9 @@ class Store(glance_store.driver.Store):
         :mount_point is path of glance data directory
         """
 
-        #Calculate total available space
-        df = processutils.execute("df", "-k",
-                                  mount_point)[0].strip("'\n'")
-        total_available_space = int(df.split('\n')[1].split()[3]) * units.Ki
-
+        # Calculate total available space
+        stvfs_result = os.statvfs(mount_point)
+        total_available_space = stvfs_result.f_bavail * stvfs_result.f_bsize
         return max(0, total_available_space)
 
     def _find_best_datadir(self, image_size):
@@ -413,6 +560,7 @@ class Store(glance_store.driver.Store):
 
         return best_datadir
 
+    @capabilities.check
     def add(self, image_id, image_file, image_size, context=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -461,13 +609,23 @@ class Store(glance_store.driver.Store):
                 self._delete_partial(filepath, image_id)
 
         checksum_hex = checksum.hexdigest()
-        metadata = self._get_metadata()
+        metadata = self._get_metadata(filepath)
 
         LOG.debug(_("Wrote %(bytes_written)d bytes to %(filepath)s with "
                     "checksum %(checksum_hex)s"),
                   {'bytes_written': bytes_written,
                    'filepath': filepath,
                    'checksum_hex': checksum_hex})
+
+        if self.conf.glance_store.filesystem_store_file_perm > 0:
+            perm = int(str(self.conf.glance_store.filesystem_store_file_perm),
+                       8)
+            try:
+                os.chmod(filepath, perm)
+            except (IOError, OSError):
+                LOG.warn(_LW("Unable to set permission to image: %s") %
+                         filepath)
+
         return ('file://%s' % filepath, bytes_written, checksum_hex, metadata)
 
     @staticmethod
@@ -477,4 +635,4 @@ class Store(glance_store.driver.Store):
         except Exception as e:
             msg = _('Unable to remove partial image '
                     'data for image %(iid)s: %(e)s')
-            LOG.error(msg % dict(iid=iid, e=e))
+            LOG.error(msg % dict(iid=iid, e=utils.exception_to_str(e)))
